GENCCODE_ASSEMBLY_TYPE=
SO=dll
SOBJ=dll
A=lib
LIBPREFIX=
LIB_EXT_ORDER=58.dll
COMPILE=cl   -DHAVE_DLOPEN=0 -DU_HAVE_ATOMIC=1 -DU_HAVE_MMAP=0 -DU_HAVE_DIRENT_H=0 -DU_HAVE_POPEN=0 -DU_HAVE_TZNAME=0 -DU_HAVE_STRTOD_L=0  -DU_RELEASE=1 -D_CRT_SECURE_NO_DEPRECATE -DU_ATTRIBUTE_DEPRECATED= -DWIN32 -DCYGWINMSVC -Gy -MD  -MD -GL   -GF -nologo -c
LIBFLAGS=-I/c/ci/icu_1510869463787/_h_env/Library/icu/include  
GENLIB=LINK.EXE -DLL  -nologo
LDICUDTFLAGS=-base:0x4ad00000 -NOENTRY
LD_SONAME=-IMPLIB:
RPATH_FLAGS=
BIR_LDFLAGS=
AR=LIB.EXE
ARFLAGS=-nologo
RANLIB=ls -s
INSTALL_CMD=/usr/bin/install -c
